@@ -125,7 +125,7 @@
                         executeWithInstantDigest(() => {
                             scope.isLoadingFilters = true;
                         });
-                        DashboardFilters.resetGlobalFilters(scope.page.id);
+                        DashboardFilters.resetGlobalFilters(scope.page.cid);
                         return DashboardFilters.initPageFilters(scope.page, { pageFilters: pageFilters }, Object.values(scope.insightsMap))
                             .then(({ filters }) => {
                                 scope.handleFiltersChange(filters);
@@ -156,7 +156,7 @@
                 scope.FiltersPanelPosition = FiltersPanelPosition;
                 scope.FiltersPanelDirection = FiltersPanelDirection;
                 scope.isInExport = getCookie('dku_graphics_export') === 'true';
-                scope.isFilterPanelDeactivated = !!DashboardFilters.getIsFilterPanelDeactivated(scope.page.id);
+                scope.isFilterPanelDeactivated = !!DashboardFilters.getIsFilterPanelDeactivated(scope.page.cid);
 
                 DashboardPageUtils.flushCachedColumnSummaries();
                 DashboardFilters.flushDatasetLowerCaseStatus();
@@ -231,7 +231,7 @@
                             scope.page.filters = filters;
                         }
                         $rootScope.$broadcast('dashboardFiltersChanged', { currentFilters: filters, previousFilters: scope.activeFilters });
-                        const activeFilters = DashboardFilters.getActiveGlobalFilters(scope.page.id);
+                        const activeFilters = DashboardFilters.getActiveGlobalFilters(scope.page.cid);
                         if (!(activeFilters || []).length && !(scope.activeFilters || []).length) {
                             return;
                         } else {
@@ -277,7 +277,7 @@
 
                 scope.handleIsDeactivatedChange = function(isDeactivated) {
                     $timeout(() => {
-                        DashboardFilters.setIsFilterPanelDeactivated(scope.page.id, isDeactivated);
+                        DashboardFilters.setIsFilterPanelDeactivated(scope.page.cid, isDeactivated);
                         scope.activeFilters = isDeactivated ? [] : scope.filters;
                     });
                 };
