@@ -239,8 +239,8 @@ app.controller("BaseSQLDatasetController", function($scope, $stateParams, $contr
 });
 
 
-app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $controller, LoggerProvider, 
-    DataikuAPI, DatasetUtils, Dialogs, ActivityIndicator, $state) {
+    app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $controller, LoggerProvider,
+                                                            DataikuAPI, DatasetUtils, Dialogs, ActivityIndicator, $state, MonoFuture) {
     $controller("BaseSQLDatasetController", {$scope: $scope, withConsistency: true});
 
     var Logger = LoggerProvider.getLogger('datasets.sql');
@@ -427,11 +427,11 @@ app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $c
         }
         // Don't test obviously wrong stuff
         //if ($scope.dataset.params.partitioned == "true" && $scope.dataset.params.mode == "table")
-        DataikuAPI.datasets.externalSQL.test($stateParams.projectKey, $scope.dataset, 15,
+        MonoFuture($scope).wrap(DataikuAPI.datasets.externalSQL.test)($stateParams.projectKey, $scope.dataset, 15,
             listTables, testTableOrQuery, getTableMetadata).success(function (data) {
             Logger.info('Got test result');
             $scope.testing = false;
-            $scope.testResult = data;
+            $scope.testResult = data.result;
             if ($scope.testResult.queryOK && $scope.testResult.querySchema) {
                 $scope.consistency = { empty : false, result : $scope.testResult.schemaDetection };
                 $scope.consistency.kind = DatasetUtils.getKindForConsistency($scope.dataset);
@@ -492,9 +492,8 @@ app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $c
             if (getTableMetadata && $scope.testResult.table) {
                 $scope.dataset.description = $scope.testResult.table.remarks;
             }
-        }).error(function (a, b, c) {
-            $scope.testing = false;
-            setErrorInScope.bind($scope)(a,b,c);
+        }).error(function(res, httpCode, c) {
+            $scope.setErrorMessageForFuture(res, httpCode, c)
         });
     };
 
@@ -544,7 +543,7 @@ app.controller("ExternalHiveDatasetController", function($scope, $stateParams, $
 });
 
 
-app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $controller, $q, LoggerProvider, DataikuAPI, Dialogs, CreateModalFromTemplate, ActivityIndicator, DatasetTypesService) {
+    app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $controller, $q, LoggerProvider, DataikuAPI, Dialogs, CreateModalFromTemplate, ActivityIndicator, DatasetTypesService, MonoFuture) {
     $controller("BaseSQLDatasetController", {$scope: $scope, withConsistency: true});
 
     var Logger = LoggerProvider.getLogger('datasets.sql');
@@ -781,10 +780,10 @@ app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $co
         var deferred = $q.defer();
         $scope.testResult = null;
         $scope.testing = true;
-        DataikuAPI.datasets.managedSQL.test($stateParams.projectKey, $scope.dataset, 15, connectionOnly).success(function (data) {
+        MonoFuture($scope).wrap(DataikuAPI.datasets.managedSQL.test)($stateParams.projectKey, $scope.dataset, 15, connectionOnly).success(function(data) {
             Logger.info('Got test result');
             $scope.testing = false;
-            $scope.testResult = data;
+            $scope.testResult = data.result;
             
             if (!$scope.testResult) {
                  $scope.testResultCase = null;
@@ -813,9 +812,8 @@ app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $co
             }
             deferred.resolve();
 
-        }).error(function (a,b,c) {
-            $scope.testing = false;
-            setErrorInScope.bind($scope)(a,b,c);
+        }).error(function(res, httpCode, c) {
+            $scope.setErrorMessageForFuture(res, httpCode, c)
             deferred.reject();
         });
         return deferred.promise;
