@@ -779,6 +779,12 @@ app.filter('typeToColor', function($filter) {
     }
     return function(type, background) {
         if (!type) return "";
+        
+        // Special case for GEN_AI_MODEL and AI_AGENT
+        if (type === 'GEN_AI_MODEL' || type === 'AI_AGENT') {
+            return 'agent-tool';
+        }
+        
         const recipeColor = $filter('recipeTypeToColor')(type);
         if (recipeColor) {
             return recipeColor;
