@@ -134,7 +134,7 @@
             const colorContext = {
                 chartData,
                 colorOptions: ChartColorUtils.getChartColorOptions(chartDef),
-                defaultLegendDimension: ChartColorUtils.getDefaultLegendDimension(chartDef),
+                defaultLegendDimension: ChartColorUtils.getDefaultLegendDimension(chartDef, chartData),
                 colorSpec,
                 chartHandler,
                 ignoreLabels,
@@ -179,7 +179,7 @@
                 }
             });
 
-            ChartLegendUtils.createLegend($container, chartDef, chartData, chartHandler, colorSpec, colorScale, undefined, ignoreLabels).then(function() {
+            ChartLegendUtils.createLegend($container, chartDef, chartData, chartHandler, colorSpec, colorScale, { ignoreLabels }).then(function() {
                 $timeout(() => {
                     const $svgs = createSVGs($container, chartData, chartDef, zoomContext && ChartDimension.isInteractiveChart(chartDef), ignoreLabels);
 
@@ -248,7 +248,8 @@
                         measureFormatters: measureFormatters,
                         chartData: chartData,
                         xSpec: axisSpecs && axisSpecs['x'],
-                        ySpecs: ySpecs
+                        ySpecs: ySpecs,
+                        colorSpec
                     };
 
                     $container.data('previous-tooltip', tooltips);
