@@ -4,6 +4,7 @@ import logging
 import uuid
 from asyncio import CancelledError
 from typing import AsyncIterator, List, Union, Optional, Dict, Any
+from pydantic import TypeAdapter
 
 from vllm.entrypoints.openai.protocol import (
     ChatCompletionRequest,
@@ -16,30 +17,40 @@ from vllm.entrypoints.openai.protocol import (
     ResponseFormat,
     JsonSchemaResponseFormat,
     ChatCompletionResponseChoice,
-    ToolCall
+    ToolCall,
+    EmbeddingCompletionRequest,
+    EmbeddingResponse,
 )
 from vllm.entrypoints.chat_utils import (
     ChatCompletionMessageParam,
     ChatCompletionContentPartParam,
-    ChatCompletionMessageToolCallParam, # type: ignore
-    ChatCompletionToolMessageParam, # type: ignore
+    ChatCompletionMessageToolCallParam,  # type: ignore
+    ChatCompletionToolMessageParam,  # type: ignore
 )
 
 from dataiku.huggingface.chat_template import ChatTemplateRenderer
-from dataiku.huggingface.types import ProcessSinglePromptCommand, StreamResponseChunkOrFooter
-from dataiku.huggingface.types import ProcessSinglePromptResponseText
-from dataiku.huggingface.types import StreamedCompletionResponseFooter
-from dataiku.huggingface.types import DetailedLogProb
-from dataiku.huggingface.types import UsageData
-from dataiku.huggingface.types import FinishReason
-from dataiku.huggingface.types import ProcessSinglePromptResponseTextFull
-from dataiku.huggingface.types import ChatMessage
-from dataiku.huggingface.types import CompletionSettings
-from dataiku.huggingface.types import ToolChoice
-from dataiku.huggingface.types import FunctionToolCall
-from dataiku.llm.python.types import SingleCompletionQuery
+from dataiku.huggingface.types import (
+    ProcessSinglePromptCommand,
+    StreamResponseChunkOrFooter,
+    ProcessSinglePromptResponseText,
+    ProcessSingleEmbeddingCommand,
+    ProcessSingleEmbeddingResponse,
+    StreamedCompletionResponseFooter,
+    DetailedLogProb,
+    UsageData,
+    FinishReason,
+    ProcessSinglePromptResponseTextFull,
+    ChatMessage,
+    CompletionSettings,
+    ToolChoice,
+    FunctionToolCall,
+    SingleCompletionQuery,
+    EmbeddingQuery,
+    EmbeddingSettings,
+)
 
-def _create_json_instruction_message(schema:Optional[Dict[str, Any]]) -> ChatMessage:
+
+def _create_json_instruction_message(schema: Optional[Dict[str, Any]]) -> ChatMessage:
     if schema is not None:
         return {
             "role": "system",
@@ -51,14 +62,16 @@ def _create_json_instruction_message(schema:Optional[Dict[str, Any]]) -> ChatMes
             "content": "Answer in JSON format.",
         }
 
+
 logger = logging.getLogger(__name__)
 
-def dss_to_oai_request(
+
+def dss_to_oai_completion_request(
     request: ProcessSinglePromptCommand,
     chat_template_renderer: ChatTemplateRenderer,
     json_constraints_in_prompt: bool,
-    guided_decoding_backend: str,
-    tools_supported: bool
+    tools_supported: bool,
+    reasoning_supported: bool,
 ) -> ChatCompletionRequest:
     params = request["settings"]
     messages = request["query"]["messages"]
@@ -122,6 +135,7 @@ def dss_to_oai_request(
         temperature=temperature,
         presence_penalty=presence_penalty,
         frequency_penalty=frequency_penalty,
+        include_reasoning=reasoning_supported, # This parameter is only used by GPT-OSS to return/not return the reasoning in the deltas
         logit_bias=logit_bias,
         logprobs=logprobs,
         top_logprobs=top_logprobs,
@@ -132,7 +146,6 @@ def dss_to_oai_request(
         stop=stop,
         seed=None,
         response_format=response_format,
-        guided_decoding_backend=guided_decoding_backend,
         **({
             'tools': tools,
             'tool_choice': tool_choice,
@@ -141,6 +154,23 @@ def dss_to_oai_request(
     )
 
 
+def dss_to_oai_embedding_request(
+    request: ProcessSingleEmbeddingCommand,
+) -> EmbeddingCompletionRequest:
+    if "text" not in request["query"] or request["query"]["text"] is None:
+        raise ValueError("Embedding query should contain a 'text' field")
+
+    truncate_prompt_tokens = None
+    if request["settings"]["textOverflowMode"] == "TRUNCATE":
+        truncate_prompt_tokens = -1
+
+    return EmbeddingCompletionRequest(
+        model="model",
+        input=request["query"]["text"],
+        truncate_prompt_tokens=truncate_prompt_tokens,
+    )
+
+
 def dss_to_oai_message(message: ChatMessage) -> ChatCompletionMessageParam:
     if "content" in message and message["content"] is not None:
         return {
@@ -186,10 +216,10 @@ def dss_to_oai_message(message: ChatMessage) -> ChatCompletionMessageParam:
                 raise ValueError("Tool call must contain a 'function' key")
             oai_tool_calls.append({
                 "type": "function",
-                "id": tool_call["id"] if "id" in tool_call else "",
+                "id": (tool_call.get("id") or "") if "id" in tool_call else "",
                 "function": {
                     "arguments": tool_call["function"]["arguments"] if "arguments" in tool_call["function"] else "",
-                    "name": tool_call["function"]["name"] if "name" in tool_call["function"] else "",
+                    "name": (tool_call["function"]["name"] or "") if "name" in tool_call["function"] else "",
                 },
             })
         return {
@@ -245,6 +275,7 @@ def oai_to_dss_finish_reason(oai_finish_reason: Optional[str]) -> FinishReason:
     else:
         return "unknown"
 
+
 def dss_to_oai_tool_choice(dss_tool_choice: ToolChoice):
     if dss_tool_choice["type"] == "auto":
         return "auto"
@@ -258,6 +289,7 @@ def dss_to_oai_tool_choice(dss_tool_choice: ToolChoice):
             "function": {"name": dss_tool_choice["name"]}
         }
 
+
 def _single_oai_tool_call_to_dss_tool_call(oai_tool_call: ToolCall) -> FunctionToolCall:
     return {
         "id": oai_tool_call.id,
@@ -268,19 +300,35 @@ def _single_oai_tool_call_to_dss_tool_call(oai_tool_call: ToolCall) -> FunctionT
         }
     }
 
+
 def oai_to_dss_tool_calls(choice: ChatCompletionResponseChoice) -> List[FunctionToolCall]:
     tool_calls = choice.message.tool_calls
     return [_single_oai_tool_call_to_dss_tool_call(oai_tool_call) for oai_tool_call in tool_calls]
 
+def oai_to_dss_reasoning_artifacts(choice: ChatCompletionResponseChoice):
+    return [{
+        'id': str(uuid.uuid4()),
+        'type': 'REASONING',
+        'parts': [{
+            'type': 'TEXT',
+            'text': choice.message.reasoning_content,
+            'index': choice.index
+        }]
+    }]
+
+def oai_to_dss_error_response(
+    response: ErrorResponse
+) -> BaseException:
+    if "Client disconnected" in response.error.message:
+        return CancelledError(response.error.message)
+    return Exception(response.error.message)
 
 
-async def oai_to_dss_response(
+async def oai_to_dss_completion_response(
     response: Union[ChatCompletionResponse, AsyncIterator[str], ErrorResponse],
 ) -> AsyncIterator[ProcessSinglePromptResponseText]:
     if isinstance(response, ErrorResponse):
-        if "Client disconnected" in response.message:
-            raise CancelledError(response.message)
-        raise Exception(response.message)
+        raise oai_to_dss_error_response(response)
 
     if isinstance(response, ChatCompletionResponse):
         yield oai_to_dss_full_response(response)
@@ -289,6 +337,21 @@ async def oai_to_dss_response(
             yield resp_chunk
 
 
+def oai_to_dss_embedding_response(
+    response: Union[EmbeddingResponse, ErrorResponse],
+) -> ProcessSingleEmbeddingResponse:
+    if isinstance(response, ErrorResponse):
+        raise oai_to_dss_error_response(response)
+
+    data = response.data[0]
+    dss_response: ProcessSingleEmbeddingResponse = {
+        "embedding": list(data.embedding),
+        "promptTokens": response.usage.prompt_tokens,
+    }
+
+    return dss_response
+
+
 def oai_to_dss_full_response(
     response: ChatCompletionResponse,
 ) -> ProcessSinglePromptResponseTextFull:
@@ -304,10 +367,15 @@ def oai_to_dss_full_response(
     dss_response["usage"] = oai_to_dss_usage(response.usage)
     dss_response["finishReason"] = oai_to_dss_finish_reason(choice.finish_reason)
     dss_response["toolCalls"] = oai_to_dss_tool_calls(choice)
+    if choice.message.reasoning_content:
+        dss_response["artifacts"] = oai_to_dss_reasoning_artifacts(choice)
 
     return dss_response
 
 
+ChunkType = TypeAdapter(Union[ChatCompletionStreamResponse, ErrorResponse])
+
+
 async def oai_to_dss_stream_response(
     response: AsyncIterator[str],
 ) -> AsyncIterator[StreamResponseChunkOrFooter]:
@@ -315,12 +383,23 @@ async def oai_to_dss_stream_response(
     prefix = "data: "
     oai_usage = None
     oai_finish_reason = None
+    reasoning_id = str(uuid.uuid4())
     async for chunk in response:
         if chunk.startswith(prefix) and len(chunk) > len(prefix):
             chunk_data = chunk[len(prefix) :]
             if chunk_data.startswith("[DONE]"):
                 break
-            parsed_chunk = ChatCompletionStreamResponse.model_validate_json(chunk_data)
+            try:
+                parsed_chunk = ChunkType.validate_json(chunk_data)
+            except:
+                logger.error("Unexpected chunk data: %s" % chunk_data)
+                raise
+
+            if isinstance(parsed_chunk, ErrorResponse):
+                if "Client disconnected" in parsed_chunk.error.message:
+                    raise CancelledError(parsed_chunk.error.message)
+                raise Exception(parsed_chunk.error.message)
+
             if parsed_chunk.usage:
                 oai_usage = parsed_chunk.usage
 
@@ -341,6 +420,17 @@ async def oai_to_dss_stream_response(
                 if dss_logprobs is not None:
                     dss_chunk["chunk"]["logProbs"] = dss_logprobs
 
+                if choice.delta and choice.delta.reasoning_content is not None:
+                    dss_chunk["chunk"]["artifacts"] = [{
+                        'id': reasoning_id,
+                        'type': 'REASONING',
+                        'parts': [{
+                            'type': 'TEXT',
+                            'text': choice.delta.reasoning_content,
+                            'index': choice.index
+                        }]
+                    }]
+
                 if choice.delta and choice.delta.tool_calls is not None:
                     dss_chunk["chunk"]["toolCalls"] = []
                     for delta_tool_call in choice.delta.tool_calls:
@@ -355,7 +445,6 @@ async def oai_to_dss_stream_response(
                                 }
                             }
                             dss_chunk["chunk"]["toolCalls"].append(tool_call)
-
                 yield dss_chunk
 
     dss_footer: StreamedCompletionResponseFooter = {
@@ -372,7 +461,7 @@ async def oai_to_dss_stream_response(
     yield { "footer": dss_footer }
 
 
-def generate_dummy_dss_request(
+def generate_dummy_dss_completion_request(
     prompt: str, max_tokens=64
 ) -> ProcessSinglePromptCommand:
     query: SingleCompletionQuery = {
@@ -396,3 +485,21 @@ def generate_dummy_dss_request(
         "type": "process-completion-query",
     }
     return request
+
+
+def generate_dummy_dss_embedding_request(
+    text: str
+) -> ProcessSingleEmbeddingCommand:
+    query: EmbeddingQuery = {
+        "text": text,
+    }
+    settings: EmbeddingSettings = {
+        "textOverflowMode": "TRUNCATE",
+    }
+    request: ProcessSingleEmbeddingCommand = {
+        "id": str(uuid.uuid4()),
+        "query": query,
+        "settings": settings,
+        "type": "process-embedding-query",
+    }
+    return request
